import {
  queryBackwash,
  queryBackwashList,
  queryDesignNob,
  queryDesignNobList,
  queryDesignWash,
  queryDesignWashList,
  queryDrug,
  queryDrugList,
  queryMembrane,
  queryMembraneConditions,
  queryMembraneList,
  queryProjectConfig,
} from '@/services/SmartOps';
import { history, useParams, useRequest } from '@umijs/max';
import {
  Button,
  Col,
  DatePicker,
  Empty,
  Form,
  Icon,
  List,
  Modal,
  Row,
  Spin,
} from 'antd';
import * as echarts from 'echarts';
import dayjs from 'dayjs';
import { useEffect, useMemo, useRef, useState } from 'react';
import Panel from './Panel';
import styles from './SimulateDetail.less';

const { RangePicker } = DatePicker;

const TYPE = {
  td_uf: {
    name: '超滤膜',
    device: (params) => queryMembraneList({ ...params, type: 'uf' }),
    chart: (params) => queryMembrane({ ...params, type: 'uf' }),
  },
  td_mf: {
    name: '微滤膜',
    device: (params) => queryMembraneList({ ...params, type: 'mf' }),
    chart: (params) => queryMembrane({ ...params, type: 'mf' }),
  },
  td_nf: {
    name: '纳滤膜',
    device: (params) => queryMembraneList({ ...params, type: 'nf' }),
    chart: (params) => queryMembrane({ ...params, type: 'nf' }),
  },
  td_ro: {
    name: '反渗透膜',
    device: (params) => queryMembraneList({ ...params, type: 'ro' }),
    chart: (params) => queryMembrane({ ...params, type: 'ro' }),
  },
  tdr_pac: {
    name: '絮凝剂投加',
    device: (params) => queryDrugList({ ...params, type: 'pac' }),
    chart: (params) => queryDrug({ ...params, type: 'pac' }),
  },
  tdr_hci: {
    name: 'HCI投加',
    device: (params) => queryDrugList({ ...params, type: 'hci' }),
    chart: (params) => queryDrug({ ...params, type: 'hci' }),
  },
  tdr_nob: {
    name: '非氧化杀菌剂投加',
    device: (params) => queryDrugList({ ...params, type: 'nob' }),
    chart: (params) => queryDrug({ ...params, type: 'nob' }),
  },
  tt_backwash: {
    name: '反冲洗记录',
    device: queryBackwashList,
    chart: queryBackwash,
  },
  tt_wash: {
    name: '大水量冲洗记录',
    device: queryDesignWashList,
    chart: queryDesignWash,
  },
  tt_nob: {
    name: '非氧化杀菌记录',
    device: queryDesignNobList,
    chart: queryDesignNob,
  },
};
const SimulateDetail = (props) => {
  const { projectId } = props;
  const [active, setActive] = useState();
  const [current, setCurrent] = useState();

  const { data } = useRequest(queryProjectConfig, {
    defaultParams: [projectId],
    onSuccess(data) {
      setActive(data[0]);
    },
  });

  const onLeftClick = () => {
    document.getElementById('content').scrollLeft -= 150;
  };

  const onRightClick = () => {
    document.getElementById('content').scrollLeft += 150;
  };

  return (
    <Panel
      title={'模拟记录'}
      style={{ width: '100%' }}
      btns={
        <div className={styles.tabs}>
          <div className={styles.left} onClick={onLeftClick}></div>
          <div id="content" className={styles.content}>
            {data?.map((item) => (
              <div
                key={item}
                className={`${styles.item} ${
                  active == item ? styles.active : ''
                }`}
                onClick={() => {
                  setActive(item);
                  setCurrent(null);
                }}
              >
                {TYPE[item]?.name}
              </div>
            ))}
          </div>
          <div className={styles.right} onClick={onRightClick}></div>
        </div>
      }
    >
      <div className={styles.box}>
        <ListContent
          active={active}
          projectId={projectId}
          current={current}
          onClick={setCurrent}
        />
        <ChartContent active={active} projectId={projectId} current={current} />
      </div>
    </Panel>
  );
};

const ListContent = (props) => {
  const { current, onClick, projectId, active } = props;

  const { data, loading, run } = useRequest(
    () => {
      let params = {
        page: 1,
        page_size: 99999,
        project_id: projectId,
      };
      return TYPE[active]?.device(params);
    },
    {
      manual: true,
      onSuccess(data) {
        if (data.list?.[0]) {
          onClick(data.list[0]);
        } else {
          onClick(null);
        }
      },
    },
  );

  useEffect(() => {
    if (active) run();
  }, [active]);

  return (
    <div className={styles.listBox}>
      <div className={styles.title}>设备列表</div>
      <List
        split={false}
        className={styles.list}
        dataSource={data?.list || []}
        renderItem={(item, idx) => (
          <List.Item
            className={
              item == current ? styles.listItemActive : styles.listItem
            }
            style={
              idx % 2 == 0
                ? {
                    background:
                      'linear-gradient(to right, rgba(153, 231, 255, 0.1), rgba(96, 168, 255, 0.1))',
                  }
                : {
                    background:
                      'linear-gradient(to right, rgba(153, 231, 255, 0.2), rgba(96, 168, 255, 0.2))',
                  }
            }
            onClick={() => onClick(item)}
          >
            {item.device_code}
          </List.Item>
        )}
      />
    </div>
  );
};

const ChartContent = (props) => {
  const { current, projectId, active } = props;
  const [visible, setVisible] = useState(false);
  const [params, setParams] = useState(null);
  const [time, setTime] = useState([dayjs().startOf('day'), dayjs()]);
  const timerRef = useRef({
    s_time: time[0].format('YYYY-MM-DD HH:mm:ss'),
    e_time: time[1].format('YYYY-MM-DD HH:mm:ss'),
  });
  const domRef = useRef(null);
  const chartRef = useRef(null);

  const { data, loading, run } = useRequest(
    () => {
      let params = {
        device_code: current.device_code,
        page: 1,
        page_size: 9999,
        project_id: projectId,
        ...timerRef.current,
      };
      return TYPE[active].chart(params);
    },
    {
      manual: true,
      onSuccess(data) {
        chartRef.current.clear();
        let options = getOption(data.list, active);
        chartRef.current.setOption(options, true);
        chartRef.current.resize();
      },
    },
  );

  const optimization = useMemo(() => {
    const result = data?.list?.filter((item) => item.optimization);
    if (result?.length > 0) {
      const lastItem = result[result.length - 1].optimization;
      const valueList = Object.values(lastItem).map((item) => item.remark);
      return valueList.join('，');
    }
    // if (data?.list?.[0]?.optimization?.peb_interval) {
    //   return data.list[0].optimization.peb_interval.remark;
    // }
    return '';
  }, [data]);

  const searchTime = (type) => {
    let time = [dayjs().startOf(type), dayjs()];
    onSearch?.(time);
  };

  const onSearch = (time) => {
    if (time && time.length == 2) {
      let s_time, e_time;
      s_time = time[0].format('YYYY-MM-DD HH:mm:ss');
      e_time = time[1].format('YYYY-MM-DD HH:mm:ss');

      timerRef.current = { s_time, e_time };
      setTime(time);
      run();
    }
  };

  useEffect(() => {
    chartRef.current = echarts.init(domRef.current);

    return () => {
      chartRef.current.dispose();
    };
  }, []);

  useEffect(() => {
    if (current?.device_code && active) {
      run();
    } else {
      chartRef.current.clear();
    }
  }, [current, active]);

  return (
    <div className={styles.chartBox}>
      <div className={styles.title}>
        优化建议
        {active == 1 && (
          <Icon
            type="area-chart"
            style={{ float: 'right', lineHeight: '56px', marginRight: 20 }}
            onClick={() => setVisible(true)}
          />
        )}
      </div>
      {/* <SearchModule style={{ margin: '20px 0' }} onSearch={onSearch} type={2} /> */}
      <Form layout="inline" style={{ margin: '10px 0' }}>
        <Row gutter={24}>
          <Col span={14}>
            <Form.Item
              label="时间"
              labelCol={{ span: 4 }}
              wrapperCol={{ span: 20 }}
            >
              <RangePicker
                allowClear={false}
                value={time}
                onChange={onSearch}
              ></RangePicker>
            </Form.Item>
          </Col>
          <Col span={10}>
            <div>
              <Button
                type="primary"
                onClick={() => searchTime('day')}
                style={{ marginRight: 10 }}
              >
                今日
              </Button>
              <Button
                type="primary"
                onClick={() => searchTime('week')}
                style={{ marginRight: 10 }}
              >
                本周
              </Button>
              <Button type="primary" onClick={() => searchTime('month')}>
                本月
              </Button>
            </div>
          </Col>
        </Row>
      </Form>
      <Spin spinning={loading}>
        {!loading && !data?.list && <Empty style={{ height: 195 }} />}
        <div
          ref={domRef}
          style={{ height: '100%', display: !data?.list ? 'none' : 'block' }}
        />
      </Spin>
      {optimization && (
        <div
          style={{
            fontSize: 22,
            // color: '#fff',
            textIndent: 30,
            padding: '10px 0',
            flexShrink: 1,
          }}
        >
          优化建议：{optimization}
        </div>
      )}

      <MembraneModal
        visible={visible}
        onCancel={() => setVisible(false)}
        params={params}
      />
    </div>
  );
};

function getOption(data = [], active) {
  let formatter,
    yAxisName = '',
    series = [],
    xAxis = [];
  var data1 = [],
    data2 = [],
    data3 = [],
    data4 = [];
  switch (active) {
    case 'tt_backwash':
      yAxisName = '清洗周期/Min';
      formatter = (params) => {
        let item = data[params[0].dataIndex];
        let content = '';
        if (item.bw_type == 1) {
          // PEB
          content += 'PEB 反洗开始时间：' + item.peb_st;
          content += '<br />PEB 反洗结束时间：' + item.peb_et;
        } else {
          // CEB
          content += 'CEB 反洗开始时间：' + item.ceb_st;
          content += '<br />CEB 反洗结束时间：' + item.ceb_et;
          content += '<br />CEB 清洗剂浓度' + item.ceb_ppm;
        }
        return content;
      };
      data?.forEach((item) => {
        if (item.bw_type == 1) {
          // 实际冲洗
          data1.push(Math.ceil(item.peb_interval / 60));
          // TODO:冲洗
          data2.push(Math.ceil(item.peb_interval / 60));
          data3.push(null);
          data4.push(null);
          xAxis.push(dayjs(item.peb_st).format('YYYY-MM-DD HH:mm:ss'));
        } else {
          data1.push(null);
          data2.push(null);
          data3.push(Math.ceil(item.peb_interval / 60));
          // TODO:模拟冲洗
          data4.push(Math.ceil(item.peb_interval / 60));
          xAxis.push(dayjs(item.ceb_st).format('YYYY-MM-DD HH:mm:ss'));
        }
      });
      series = [
        {
          name: '物理实际冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data1,
        },
        {
          name: '物理模拟冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data2,
        },
        {
          name: '化学实际冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data3,
        },
        {
          name: '化学模拟冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data4,
        },
      ];
      break;
    case 'tt_wash':
      yAxisName = '清洗周期/Min';
      formatter = (params) => {
        let item = data[params[0].dataIndex];
        let content = '';
        content += '反洗开始时间：' + item.st;
        content += '<br />反洗结束时间：' + item.et;
        return content;
      };
      data?.forEach((item) => {
        // 实际冲洗
        data1.push(Math.ceil(item.interval / 60));
        // TODO:模拟冲洗
        data2.push(Math.ceil(item.interval / 60));
        xAxis.push(dayjs(item.st).format('YYYY-MM-DD HH:mm:ss'));
      });
      series = [
        {
          name: '实际冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data1,
        },
        {
          name: '模拟冲洗',
          type: 'bar',
          barMaxWidth: '20px',
          data: data2,
        },
      ];
      break;
    case 'tt_nob':
      yAxisName = '杀菌周期/Min';
      formatter = (params) => {
        let item = data[params[0].dataIndex];
        let content = '';
        content += '杀菌开始时间：' + (item.st || '-');
        content += '<br />杀菌结束时间：' + (item.et || '-');
        return content;
      };
      var data1 = [],
        data2 = [];
      data?.forEach((item) => {
        // 实际冲洗
        data1.push(Math.ceil(item.interval / 60));
        // TODO:模拟冲洗
        data2.push(Math.ceil(item.interval / 60));
        xAxis.push(dayjs(item.st).format('YYYY-MM-DD HH:mm:ss'));
      });
      series = [
        {
          name: '实际',
          type: 'bar',
          barMaxWidth: '20px',
          data: data1,
        },
        {
          name: '模拟',
          type: 'bar',
          barMaxWidth: '20px',
          data: data2,
        },
      ];
      break;
    case 'tdr_hci':
    case 'tdr_nob':
    case 'tdr_pac':
      yAxisName = '投加量';
      // formatter = params => {
      //   let content = '';
      //   let item = data[params[0].dataIndex];

      //   content += item.c_time;
      //   content += '<br />最高加药浓度' + item.dosh + 'g/m3';
      //   content += '<br />最低加药浓度：' + item.dosl + 'g/m3';
      //   content += '<br />最高加药浊度：' + item.tubh + 'g/m3';
      //   content += '<br />最低加药浊度：' + item.tubl + 'g/m3';
      //   content += '<br />实际进水浊度：' + item.tubr;
      //   return content;
      // };
      data?.forEach((item) => {
        // 实际冲洗
        data1.push(Math.ceil(item.fr / 60));
        // TODO:模拟冲洗
        data2.push(Math.ceil(item.fcoa / 60));
        xAxis.push(dayjs(item.c_time).format('YYYY-MM-DD HH:mm:ss'));
      });
      series = [
        {
          name: '实际物理投加量',
          type: 'bar',
          barMaxWidth: '20px',
          data: data1,
        },
        {
          name: '理论物理投加量',
          type: 'bar',
          barMaxWidth: '20px',
          data: data2,
        },
      ];
      break;

    case 'td_uf':
    case 'td_mf':
    case 'td_nf':
      yAxisName = '渗透率';
      data?.forEach((item) => {
        // 实际跨膜压差
        data1.push(item.permeability);
        // 模拟跨膜压差
        data2.push(item.std_permeability);
        xAxis.push(dayjs(item.c_time).format('YYYY-MM-DD HH:mm:ss'));
      });
      series = [
        {
          name: '实际渗透率',
          type: 'line',
          data: data1,
          showSymbol: false,
        },
        {
          name: '模拟渗透率',
          type: 'line',
          data: data2,
          showSymbol: false,
        },
      ];
      break;
    case 'td_ro':
      yAxisName = '跨膜压差';
      data?.forEach((item) => {
        // 实际跨膜压差
        data1.push(item.extend['1st_Stage_DP']);
        data2.push(item.extend['2nd_Stage_DP']);
        // 模拟跨膜压差
        data3.push(item.stabilize_extend['1st_Stage_DP']);
        data4.push(item.stabilize_extend['2nd_Stage_DP']);
        xAxis.push(dayjs(item.c_time).format('YYYY-MM-DD HH:mm:ss'));
      });
      series = [
        {
          name: '实际一段跨膜压差',
          type: 'line',
          data: data1,
          showSymbol: false,
        },
        {
          name: '实际二段跨膜压差',
          type: 'line',
          data: data2,
          showSymbol: false,
        },
        {
          name: '模拟一段跨膜压差',
          type: 'line',
          data: data3,
          showSymbol: false,
        },
        {
          name: '模拟二段跨膜压差',
          type: 'line',
          data: data4,
          showSymbol: false,
        },
      ];
      break;
  }

  const option = {
    color: ['#FFC800', '#30EDFD', '#4096ff', '#ff4d4f', '#ffa940'],
    tooltip: {
      trigger: 'axis',
      axisPointer: {
        type: 'shadow',
      },
      formatter,
    },
    legend: {
      textStyle: {
        // color: '#fff',
        fontSize: 18,
      },
    },
    grid: {
      left: '3%',
      right: '4%',
      bottom: '3%',
      containLabel: true,
    },
    xAxis: {
      type: 'category',
      data: xAxis,
      axisLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      splitLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      axisLabel: {
        // color: '#fff',
      },
    },
    yAxis: {
      name: yAxisName,
      type: 'value',
      boundaryGap: [0, 0.01],
      axisLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      splitLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      axisLabel: {
        // color: '#fff',
      },
    },
    series,
  };

  return option;
}

const MembraneModal = (props) => {
  const { visible, onCancel, params } = props;
  const domRef = useRef(null);
  const chartRef = useRef(null);

  const { run, loading } = useRequest(queryMembraneConditions, {
    manual: true,
    onSuccess(data) {
      console.log(data);
      let options = getMembraneOption(data.list);
      chartRef.current.setOption(options, true);
    },
  });

  useEffect(() => {
    if (visible) {
      if (!chartRef.current) {
        chartRef.current = echarts.init(document.getElementById('chart'));
        run(params);
      }
    }
  }, [visible]);

  return (
    <Modal
      forceRender
      title="渗透率图表"
      visible={visible}
      onCancel={onCancel}
      footer={null}
    >
      <Spin spinning={loading}>
        <div id="chart" style={{ height: '60vh' }} />
      </Spin>
    </Modal>
  );
};
function getMembraneOption(data = []) {
  const option = {
    color: ['#FFC800', '#30EDFD', '#4096ff', '#ff4d4f', '#ffa940'],
    tooltip: {
      trigger: 'axis',
      axisPointer: {
        type: 'shadow',
      },
    },
    legend: {
      textStyle: {
        // color: '#fff',
        fontSize: 18,
      },
    },
    grid: {
      left: '3%',
      right: '4%',
      bottom: '3%',
      containLabel: true,
    },
    xAxis: {
      type: 'category',
      data: data.map((item) => item.c_time),
      axisLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      splitLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      axisLabel: {
        // color: '#fff',
      },
    },
    yAxis: {
      name: '渗透率',
      type: 'value',
      boundaryGap: [0, 0.01],
      axisLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      splitLine: {
        lineStyle: {
          // color: '#fff',
        },
      },
      axisLabel: {
        // color: '#fff',
      },
    },
    series: [
      {
        type: 'line',
        showSymbol: false,
        areaStyle: {
          opacity: 0.1,
        },
        type: 'line',
        smooth: true,
        data: data.map((v) => v.permeability),
      },
    ],
  };

  return option;
}

export default SimulateDetail;
